/*
 *    Sierpinski - lattice (not optimal, but it works)
 *
 *    name   :   m_pinski.c
 *    compile:   cc m_pinski.c -o m_pinski
 *    start  :	 m_pinski ?n? > daten.def
 *       ?n? = nesting 1..8
 *
 *     (c) J.Schulenburg 1996,2000,2007
 *
 * 2007 - new format
 */


#include <stdio.h>
#include <stdlib.h>

#define Bufsize 10000 /* must be greater than number of bonds */

int g[Bufsize][2];	/* 2 Buffer for bonds */
int ecke[3];		/* Aeusserste Punkte (Ecken) */
int pos[Bufsize][2];	/* (x,y) */

#define F0(txt)	{ fprintf(stderr,"\nERROR L%d: "txt"\n",__LINE__);exit(1); }

int main(int argn,char *argv[]){
  int t,tmax,d,i,j,k,s,b,site,px,py;
  tmax=3; /* MAX-Tiefe */
  if(argn==2)tmax=atoi(argv[1]);
  t=0;
  d=1;  /* anzahl kleinste dreiecke */
  s=3;  /* sites */
  b=3;  /* bonds */
  g[0][0]=0;   g[0][1]=1;	/* rechtsdrehend definiert */
  g[1][0]=1;   g[1][1]=2;
  g[2][0]=2;   g[2][1]=0;
  ecke[0]=0;  /* unten links */
  ecke[1]=1;  /* unten rechts */
  ecke[2]=2;  /* oben */
#define X 0
#define Y 1
  pos[0][X]=0;pos[0][Y]=0;
  pos[1][X]=2;pos[1][Y]=0;
  pos[2][X]=1;pos[2][Y]=1;
  for(t=1;t<tmax;t++){	/* tiefe t = Breite des Dreiecks */
    if(3*b>Bufsize)F0("OVL, enlarge Bufsize!");
    /* 1. dreieck unten links bleibt */
    for(j=0;j<b;j++)for(k=0;k<2;k++){  /* 1. dreieck unten rechts */
       site=g[j][k]; px=pos[site][X]; py=pos[site][Y];
       if(site==ecke[0]){ site=ecke[1]; }   /* unten links = unten rechts */
       else {		  site+=s-1;	     /* rest = no+s-1 */
         pos[site][X]=px+pos[ecke[1]][X];	/* eigentlich nur bei k=0 noetig */
         pos[site][Y]=py;
       }
       g[j+b][k]=site;
    }
    for(j=0;j<b;j++)for(k=0;k<2;k++){  /* 1. dreieck oben */
       site=g[j][k]; px=pos[site][X]; py=pos[site][Y];
       if(site==ecke[0]){ site=ecke[2]; }  else /* unten links */
       if(site==ecke[1]){ site=ecke[2]+s-1; }   /* unten rechts = oben+s-1 */
       else { if(site>ecke[1])site--; site+=2*s-2;
         pos[site][X]=px+pos[ecke[2]][X];
         pos[site][Y]=py+pos[ecke[2]][Y];
       }
       g[j+2*b][k]=site;
    }
    ecke[1]+=(s-1); ecke[2]+=(s-1)+(s-2); s+=3*d; b*=3; d*=3;
    printf("# nesting=%2d mini_triangles=%4d sites=%4d bonds=%4d\n",t,d,s,b);
  }
  printf("# outer edges: %d %d %d\n# bond_no s1 s2\n",ecke[0],ecke[1],ecke[2]);
  printf("#+++model+++\n"
         "# NN=Number_of_sites Nw=Number_of_bonds Nv=Number_of_bonds(2)\n"
         "%u %u 0\n"
         "# s  X  Y  Z e# U# (s=site  (X Y Z)=position  p=parameter-Nr)\n",s,b);
  for(i=0;i<s;i++)
  printf(" %2u %6.3f %6.3f  0 e3 u4\n",i,0.5*pos[i][0],0.8660254*pos[i][1]);
  printf("# Bond s1 s2 t# J# V#\n");
  for(i=0;i<b;i++)
  printf(" %3d %3d %3d  t2  j1  v0\n",i,g[i][0],g[i][1]);
  printf("#---model---\n");
  return 0; 
}

